@@ -138,11 +138,12 @@ static void rm_pcap(char * iface )
 	 next = bpc->next;
 	 pcap_close(bpc->pcap);
 	 efree(&bpc);
-	 if ( prev ) prev->next = bpc->next;
+	 if ( prev ) prev->next = next;
 	 else pcaps = next;
 	}
   prev = bpc;
-  bpc = bpc->next;
+  if ( bpc != NULL )
+   bpc = bpc->next;
  }
 }
 
@@ -937,7 +938,7 @@ int bpf_open_live(char * iface, char * filter)
  if(iface == NULL)
   iface = pcap_lookupdev(errbuf);
  
- ret = pcap_open_live(iface, 1500, 0, 100, errbuf);
+ ret = pcap_open_live(iface, 1500, 0, 1, errbuf);
  if(ret == NULL)
  {
     printf("%s\n", errbuf);	 
@@ -974,10 +975,24 @@ u_char* bpf_next_tv(int bpf, int * caplen, struct timeval * tv)
 {
   u_char * p = NULL;
   struct pcap_pkthdr head;
+  struct timeval timeout, now;
 
+  timeout.tv_sec += tv->tv_sec;
+  timeout.tv_usec += tv->tv_usec;
+  while ( timeout.tv_usec >= 1000000 ) {
+        timeout.tv_sec ++;
+        timeout.tv_usec -= 1000000;
+  }
   
- p = (u_char*)pcap_next(pcaps[bpf], &head);
- *caplen  = head.caplen;
+ do {
+  p = (u_char*)pcap_next(pcaps[bpf], &head);
+  *caplen  = head.caplen;
+  if ( p != NULL ) break;
+  gettimeofday(&now, NULL);
+ } while ( !((now.tv_sec > timeout.tv_sec) ||
+             (now.tv_sec == timeout.tv_sec && now.tv_usec >= timeout.tv_usec ) ));
+
+
  return p;
 }
 
