@@ -6,7 +6,7 @@
  *
  *       Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *    $Id: hlst.h,v 1.9 2000/12/29 13:26:36 jordan Exp $ 
+ *    $Id: hlst.h,v 1.12 2001/03/04 21:26:24 jordan Exp $ 
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -41,10 +41,12 @@ struct _hsrch {			/* walk through the list */
   unsigned      bucket_id ;	/* current bucket */
   struct _hashqueue *ntry ;	/* pointer to the next entry */
   struct _hsrch     *next ;	/* more such entries */
+
 # ifdef ENABLE_RHLST
   void (*clup)(void*)     ;	/* for remote list processing */
   void  *clup_state ;
 # endif /* ENABLE_RHLST */
+
 } hsrch ;
 
 typedef
@@ -61,14 +63,13 @@ typedef
 struct _hlst {			/* hash list descriptor */
   struct _sorter *access;	/* there might be an index on that list */
   struct _rhlst *raccess;	/* extensions */
+  int (*sorter_fn)(void*,const char*,unsigned,const char*,unsigned);
+  void *sorter_desc ;		/* custom sort of the entries */
   void *clup_state ;
   void (*clup)			/* call back destructor for entries */
-       (void*,void*,char*,unsigned) ;
+       (void*,void*,char*,unsigned);
   hash_defs       z ;		/* hash parameters for this list */
   struct _hsrch *walk ;		/* list: walk through the hash list */
-# ifdef ENABLE_RHLST
-  int         tra_cnt ;		/* transaction counter, used for caching */
-# endif /* ENABLE_RHLST */
   unsigned total_entries ;	/* number of entries in the list */
   void *bucket [1] ;		/* varable length array of buckets */
   /* varable length, pointer aligned  */
@@ -88,8 +89,11 @@ typedef struct _hsrch {char opaq;} hsrch;
 /* open/close management */
 extern hlst *create_hlst 
   (unsigned estimated_size_hint, 
+
    /* this function is called when non-empty slots are removed, if the
-      list is destroyed, the last call will be destroy(state,0,0,0) */
+      list is destroyed, the last call will be destroy(state,0,0,0). Apart
+      from the last call, the detroy function will only be called with a
+      non NULL item, */
    void (*destroy)(void *state, void* item, char* key, unsigned keylen), 
    void* state) ;
 
@@ -98,16 +102,29 @@ extern hlst *copy_hlst
    /* leave the estimated_size_hint 0 to use the parameters from the
       list to_be_copied */
    unsigned estimated_size_hint, 
-   /* the copy() function is called when an entry is to be copied and
+
+   /* The copy() function is called when an entry is to be copied and
       inserted into the copied slot - leave it empty to copy a list with
-      keys but without inserted data, only */
+      keys but without inserted data, only. This function is only used for
+      duplicating the list. Upon copy() returning NULL while errno set to
+      a non-NULL value, copying is stopped and the partly allocated list
+      is removed using destroy_hlst(). In this case, the copy_hlst() 
+      function returns NULL while errno is set as passed from the copy() 
+      function. */
    void *(*copy) (void *cpstate, void* item, char *key, unsigned keylen),
    void *cpstate,
-   /* the destroy() function is called when non-empty slots are removed, if
-      the list is destroyed, the last call will be destroy(state,0,0,0) */
+
+   /* The destroy() function is called when non-empty slots are removed,
+      if the list is destroyed, the last call will be destroy(state,0,0,0) 
+      This function is registered, internally to be used later on.Apart
+      from the last call, the detroy function will only be called with a
+      non NULL item, */
    void (*destroy)(void *state, void* item, char* key, unsigned keylen), 
    void* state) ;
 
+/* global factor to be applied internally to any estimated_size_hint */
+extern unsigned compress_hlst_index (unsigned size_hint_percentage);
+
 extern void destroy_hlst (hlst*);
 extern void   flush_hlst 
    (hlst*,
@@ -134,7 +151,7 @@ extern char    *query_key_hlst    (void **);
 extern unsigned query_keylen_hlst (void **);
 
 #ifdef ENABLE_RHLST
-/* returns the current transaction number associated with the entry */
+/* sets/returns a transaction number associated with the entry */
 extern int query_tranum_hlst (void **);
 extern int   set_tranum_hlst (void **, int);
 #endif /* ENABLE_RHLST */
@@ -142,14 +159,6 @@ extern int   set_tranum_hlst (void **, int);
 /* returns the number of elements in the argument list (might be NULL) */
 extern unsigned query_hlst_size (hlst *);
 
-#ifdef ENABLE_RHLST
-/* returns the global transaction number status */
-extern int query_tranum_count (hlst *);
-
-/* get next positive transaction number (wrap around to 1) */
-extern int  next_tranum_count (hlst *);
-#endif /* ENABLE_RHLST */
-
 /* applying a function fn () to all list elements */
 extern int for_hlst_do
   (hlst *,
@@ -171,6 +180,16 @@ extern void   sort_hlst (hlst*);
 extern void  **inx_hlst (hlst*,unsigned);
 extern void unsort_hlst (hlst*);
 
+/* custom sort */
+extern int csort_hlst
+  (hlst*,
+   /* custom comparison function for qsort */
+   int (*)(void *desc,
+	   const char  *left_key, unsigned  left_klen,
+	   const char *right_key, unsigned right_klen),
+   void *desc);
+
+
 /* statistics etc. */
 extern int  hlst_buckets (hlst*);
 extern void hlst_statistics /* statistics, fn may be NULL to use the default */
